<?php

namespace Controller;

use Service\DeleteCascadeService;
use Service\PeriodMathExpression\Period;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Utilities;
use Service\Traits\TResponse;

class Periods extends Crud
{

    use TResponse;

    protected $dataSourceName = 'periods';
    /** @var  \Service\PeriodsService */
    protected $periodsService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, $periodsService)
    {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->periodsService = $periodsService;
        $this->orderField = '';
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {

        $params = $request->getParameters();
        $periods = $this->findAllPeriods($params);
        $periods = Utilities::parseDataSourceZeedhiNext($periods);
        $periods['data'] = $this->fillEmptyAttributes($periods['data']);

        $periods = $this->validateParameters($periods, $params);

        $this->sendResponse($periods, 200);
    }

    public function getDateByDateExpression(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow()->get('datesExpression');
        $row = $this->periodsService->getDateByDateExpression($row);

        $this->sendResponse($row, 200);
    }

    /**
     * @return array
     */
    public function findAllPeriods($params)
    {
        $basePeriods = $this->periodsService->getPeriodsFromBase($params);
        $orgPeriods = $this->periodsService->getPeriodsFromOrg($params);
        $periods = array_merge($basePeriods, $orgPeriods);
        foreach ($periods as $key => &$period) {
            $startDateExpression = !empty($period['START_DATE_EXPRESSION']) ? $period['START_DATE_EXPRESSION'] : null;
            $endDateExpression = !empty($period['END_DATE_EXPRESSION']) ? $period['END_DATE_EXPRESSION'] : null;
            if ($startDateExpression || $endDateExpression) {
                try {

                    $period['interval'] = [
                        'start' => Period::getDate($startDateExpression),
                        'end' => Period::getDate($endDateExpression),
                        '_id' => $period['_id'],
                        'customized' => true
                    ];
                } catch (\Exception $e) {
                    $period['Erro'] = $e->getMessage();
                }
            } else {
                unset($period['interval']);
            }
        }
        return $periods;
    }

    private function fillEmptyAttributes($periods)
    {
        foreach ($periods as $key => $period) {
            $periods[$key]['name'] = array_key_exists('name', $period) ? $period['name'] : '';
            $periods[$key]['labelOnView'] = array_key_exists('labelOnView', $period) ? $period['labelOnView'] : '';
        }

        return $periods;
    }

    private function validateParameters($periods, $params)
    {
        if (array_key_exists("order", $params)) {
            $periodData = $periods['data'];
            $order = explode(".", $params['order']);
            $this->orderField = $order[0];
            if ($order[1] == 'asc') usort($periodData, array($this, 'comparatorAsc'));
            else usort($periodData, array($this, 'comparatorDesc'));
            $periods['data'] = $periodData;
        }

        $limit = -1;
        if (array_key_exists("limit", $params)) {
            $limit = intval($params['limit']);
            $periods['pagination']['limit'] = $limit;
            $periodData = sizeof($periods['data']) > 0 && $limit > 0 ? array_chunk($periods['data'], $limit) : array();
        }

        $page = 1;
        if (array_key_exists("page", $params)) {
            $page = intval($params['page']);
            $periods['pagination']['page'] = $page;
            if ($limit != -1 && sizeof($periodData) >= $page) $periods['data'] = $periodData[$page - 1];
        }

        return $periods;
    }

    private function comparatorAsc($object1, $object2)
    {
        return strcmp(strtolower($object1[$this->orderField]), strtolower($object2[$this->orderField]));
    }

    private function comparatorDesc($object1, $object2)
    {
        return strcmp(strtolower($object2[$this->orderField]), strtolower($object1[$this->orderField]));
    }
}
