<?php

namespace Controller;

use Service\Traits\TResponse;
use Service\DeleteCascadeService;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use \Service\WidgetData;
use \Service\ViewType;
use \Service\Hierarchy;
use \Service\Indicator;
use \Service\Dashboard;
use Service\PeriodObservation;
use Util\Utilities;

class View extends Crud {

    use TResponse;

    protected $dataSourceName = 'view';
    /** @var \Service\View $viewService */
    protected $viewService;
    /** @var \Service\WidgetData $widgetService */
    protected $widgetService;
    /** @var \Service\ViewType $viewTypeService */
    protected $viewTypeService;
    /** @var \Service\Hierarchy $hierarchyService */
    protected $hierarchyService;
    /** @var \Service\Indicator $indicatorService */
    protected $indicatorService;
    /** @var \Service\Dashboard $dashboardService */
    protected $dashboardService;
    /** @var \Service\periodObservationService $dashboardService */
    protected $periodObservationService;
    /** @var Manager $organizationManager */
    protected $organizationManager;

    const BOOLEAN_FIELDS = [
        "isMainDashboard"
    ];

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService,
        \Service\View $viewService, WidgetData $widgetService, ViewType $viewTypeService, Hierarchy $hierarchyService,
        Indicator $indicatorService, Dashboard $dashboardService, PeriodObservation $periodObservationService)
    {
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->organizationManager = $organizationMongoManager;
        $this->organizationManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->viewService = $viewService;
        $this->widgetService = $widgetService;
        $this->viewTypeService = $viewTypeService;
        $this->hierarchyService = $hierarchyService;
        $this->indicatorService = $indicatorService;
        $this->dashboardService = $dashboardService;
        $this->periodObservationService = $periodObservationService;
    }

    public function findNext(DTO\Request $request, DTO\Response $response){
        //  $filterCriteria = $request->getFilterCriteria();
        //  $conditions = $filterCriteria->getConditions();
        $params = $request->getParameters();
        $filter = new ZmartFilterCriteria($this->dataSourceName, array());

        // Filter by field
        if (array_key_exists("in", $params)) {
            foreach ($params["in"] as $paramFilter){
                if(!array_key_exists($paramFilter, $params))
                    continue;
                $operator = FilterCriteria::EQ;
                $filterValue = $params[$paramFilter];
                if ($paramFilter == "_order"){
                    $paramFilter = "order";
                    $filterValue = (int)$filterValue;
                }
                if (is_string($filterValue))
                    $filter->addCondition($paramFilter, FilterCriteria::LIKE_I, $filterValue);
                else if (is_array($filterValue)){
                    // Alguma coisa acontece no zeedhi que campos select multiple são enviados como uma matriz ao invés de um array, esta é uma solução para o caso
                    if ($paramFilter=="accessProfiles" && is_array($filterValue[0])){
                        $accessProfiles = array();
                        foreach ($filterValue as $accessProfile) {
                            array_push($accessProfiles,$accessProfile[0]);
                        }
                        $filterValue = $accessProfiles;
                    }
                    $filter->addCondition($paramFilter, FilterCriteria::IN, $filterValue);
                }
                else
                    $filter->addCondition($paramFilter, $filterValue);
            }
        }
        $conditions = $filter->getConditions();
        $conditions = Utilities::filterConditionsToBoolean($conditions, $this::BOOLEAN_FIELDS);
        $filter = Utilities::rebuildFilterCriteriaWithDifferentConditions($filter, $conditions);
        $filter->setDataSourceName($this->dataSourceName);
        $data =  $this->viewService->findBy($filter);

        // Paginação
        $total = count($data);
        $limit = array_key_exists("limit", $params) ? (int)$params["limit"] : 9999;
        $page = array_key_exists("page", $params) ? (int)$params["page"] : 1;
        $data = array_chunk($data, $limit);
        if ($page > count($data))
            $page = count($data);
        if ($page <= 0)
            $page = 1;
        if (count($data) > 0)
            $data = $data[$page-1];

        foreach ($data as &$row) {
            if(!empty($row['parent'])){
                $row['parent_front'] = $row['parent'];
            }else{
                $row['parent_front'] = null;
            }
        }
        $data = Utilities::fixParentsForTreeGrid($data,'_id','parent_front');

        $this->sendResponse(["pagination" => ["total" => $total, "page" => $page, "limit" => $limit],"data" => $data], 200);
    }

    public function loadMenuHierarchy(Request\Filter $request, Response $response) {
        try{
            $response->addDataSet(new DataSet('menuHierarchy', $this->viewService->loadMenuHierarchy()));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function save(DTO\Request $request, DTO\Response $response)
    {
        $view = !empty($request->getDataSet()->getRows()) ? $request->getDataSet()->getRows()[0] : null;
        if($view){
            $dataBefore = null;
            if(!empty($view['_id'])){
               $dataBefore = $this->viewService->findById($view['_id']);
            }
            if (!empty($view['indicators']) && is_array($view['indicators']) && Utilities::getOrganizationIdForMongo() == '5') {
                /** @var \Service\Widget $widgetService */
                $widgetService = InstanceManager::getInstance()->getService("\Service\Widget");
                foreach ($view['indicators'] as &$widget) {
                    $widget["mockPeriod"] = empty($widget["period"]) ? null : $widgetService->setPeriodFormatMock($widget["period"], $widget['indicator']);
                }
            }
            parent::save($request, $response);
            $id = $response->getDataSets()[0]->getRows()[0]['_id'];
            $this->viewService->setLog($dataBefore, $id);
        }
    }

    public function fetchWidgetData(Request\Filter $request, Response $response) {
        try {
            $data = array();
            $row = $request->getFilterCriteria()->getConditions()[0]["value"];
            if(!empty($row['viewId']) && (!empty($row['structureId']) || !empty($row['structureTypeId'])) || !empty($row['isViewPreview'])) {
                $viewId             = $row['viewId'];
                $structureId        = isset($row['structureId'])         ? $row['structureId']          : null;
                $structureTypeId    = isset($row['structureTypeId'])     ? $row['structureTypeId']      : null;
                $period             = isset($row['period'])              ? $row['period']               : null;
                $dimension          = isset($row['dimension'])           ? $row['dimension']            : null;
                $limitRows          = isset($row['limitRows'])           ? $row['limitRows']            : null;
                $facts              = isset($row['facts'])               ? $row['facts']                : array();
                $orderBy            = isset($row['orderBy'])             ? $row['orderBy']              : array();
                $drillDownFilters   = isset($row['drillDownFilters'])    ? $row['drillDownFilters']     : array();
                $groupByStructure   = isset($row['groupByStructure'])    ? $row['groupByStructure']     : null;
                $previousCondition  = isset($row['previousConditions'])  ? $row['previousConditions']   : array();
                $postConditions     = isset($row['postConditions'])      ? $row['postConditions']       : array();
                $dimensionX         = isset($row['dimensionX'])          ? $row['dimensionX']           : null;
                $dimensionY         = isset($row['dimensionY'])          ? $row['dimensionY']           : array();
                $filterOrderByTotal = isset($row['orderByTotal'])        ? $row['orderByTotal']         : null;
                $isViewPreview = !empty($row['isViewPreview'])        ? $row['isViewPreview']         : false;

                $structureId = is_array($structureId) ? $structureId : array($structureId);
                if ($period && isset($period['period'])){
                    $period = $period['period'];
                }
                if(isset($row['dashboardPreConditions'])){
                    if(empty($previousCondition)){
                        $previousCondition = $row['dashboardPreConditions'];
                    }else{
                        $previousCondition = array(
                            'operator' => '&&',
                            'id' => 1,
                            'firstValue' => $row['dashboardPreConditions'],
                            'secondValue' => $previousCondition
                        );
                    }
                }
                $data = $this->widgetService->fetchWidgetData($viewId, $structureId, $structureTypeId, $period,
                    $facts, $dimension, $orderBy, null, $limitRows, $drillDownFilters, $groupByStructure,
                    $previousCondition, $postConditions, $filterOrderByTotal, $row,$dimensionX,$dimensionY, $isViewPreview);

            }

            $response->addDataSet(new DataSet('fetchWidgetData', $data));
        } catch (\Exception $e) {
            if($e->getCode() === Exception::CODE_FAIL_CONNECTION || $e->getCode() === Exception::CODE_QUERY_EXECUTION_FAIL) {
                $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
            } else {
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function getMainDashboardViews(Request\Row $request, Response $response){
        $row = $request->getRow();
        try{
            $mainDashboard = $this->viewService->getMainDashboardViews($row["name"]);
            list($mainDashboard,$error) = $this->dashboardService->buildWidgetIndicator($mainDashboard);
            $observations = [];
            foreach($mainDashboard['indicators'] as &$indicator){
                $this->dashboardService->fixBugGaugeInAllIndicators($indicator);
                $observations = $this->pushObservations($observations,$indicator,$row);
            }
            $response->addDataSet(new DataSet('periodObservationNotification', array_values(array_unique($observations))));
            $response->addDataSet(new DataSet('mainDashboard', $mainDashboard));
            $response->addDataSet(new DataSet('error',$error));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    private function pushObservations($observations,$indicator,$row){
        $period = array();
        if(!empty($indicator["parameters"]["period"]) && empty($row["period"]["period"])){
            $period = $indicator["parameters"]["period"];
        } else if(!empty($row["period"]["period"])){
            $period = $row["period"]["period"];
        }
        $observations = array_merge($observations,$this->periodObservationService->getObservationsByDateAndIndicatorAndDataSource($period,$indicator["indicator"],$indicator["dataSourceId"]));
        return $observations;
    }

    public function getGenericDashboardsViews(Request\Row $request, Response $response){
        $row = $request->getRow();
        try{
            $isViewPreview = !empty($row["isViewPreview"]) ? true : false;
            if($isViewPreview){
                $importDatabase = Utilities::getImportDefaultDatabase();
                $this->organizationManager->createDatabase($importDatabase);
            }
            $genericDashboard = $this->viewService->getGenericDashboardViews($row["name"]);
            list($genericDashboard, $error) = $this->dashboardService->buildWidgetIndicator($genericDashboard, null, $isViewPreview);
            $observations = [];
            if(!empty($genericDashboard['indicators'])){
                foreach($genericDashboard['indicators'] as &$indicator){
                    if(isset($indicator['legendClosed'])){
                        $chartLegendClosedValue  = $indicator['legendClosed'];
                        unset($indicator['legendClosed']);
                        $indicator['chartLegendClosed']  = $chartLegendClosedValue;
                    }
                    $this->dashboardService->fixBugGaugeInAllIndicators($indicator);
                    $observations = $this->pushObservations($observations,$indicator,$row);
                }
            }
            $response->addDataSet(new DataSet('periodObservationNotification', array_values(array_unique($observations))));
            $response->addDataSet(new DataSet('mainDashboard', $genericDashboard));
            $response->addDataSet(new DataSet('error',$error));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}
